package de.zucizu.ledlights;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;

/**
 * 
 * This window is for configuration purposes.
 * 
 * @author zucizu
 * 
 */
public class ConfigFrame extends JFrame {

	private Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

	/**
	 * Width of screen.
	 */
	private int screenWidth = (int) screenSize.getWidth();

	/**
	 * Height of screen.
	 */
	private int screenHeight = (int) screenSize.getHeight();

	/**
	 * Last regular mouse position.
	 */
	private Point mousePos = new Point(0, 0);

	/**
	 * A screen shot of the screen.
	 */
	private BufferedImage background;

	/**
	 * The current index in the positions array.
	 */
	private int currentIndex = 0;

	/**
	 * Holds all necessary positions of the areas.
	 */
	private Point[] positions = new Point[4];

	/**
	 * Created by Eclipse.
	 */
	private static final long serialVersionUID = -6511305716423056584L;

	/**
	 * Creates a new full screen frame to test the LED lights.
	 */
	public ConfigFrame(final AmbientLED mainApp) {
		super();

		this.background = mainApp.getScreenShot();

		// Full screen.
		this.setSize(this.screenSize);
		this.setUndecorated(true);

		// We need to react on user inputs.
		this.addMouseMotionListener(new MouseMotionListener() {

			@Override
			public void mouseDragged(MouseEvent arg0) {
			}

			/**
			 * The mouse position is being updated, if the user has made a
			 * regular input.
			 */
			@Override
			public void mouseMoved(MouseEvent evt) {
				int old = currentIndex - 1;
				// Only to the lower right side!
				if (currentIndex % 2 == 1
						&& (evt.getPoint().getX() < positions[old].getX() || evt
								.getPoint().getY() < positions[old].getY())) {
					return;
				}
				// Only half of the screen is allowed.
				mousePos = (evt.getPoint().getX() > screenWidth / 2) ? new Point(
						(int) (screenWidth / 2), (int) evt.getPoint().getY())
						: evt.getPoint();
				repaint();
			}

		});

		this.addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent arg0) {
			}

			@Override
			public void mousePressed(MouseEvent arg0) {
			}

			@Override
			public void mouseExited(MouseEvent arg0) {
			}

			@Override
			public void mouseEntered(MouseEvent arg0) {
			}

			/**
			 * If the user clicks, the last regular mouse position is being
			 * saved into our array and the index is being incremented. If all
			 * positions are being saved the window can close itself.
			 */
			@Override
			public void mouseClicked(MouseEvent arg0) {
				if (currentIndex < positions.length) {
					positions[currentIndex++] = mousePos;
				}
				if (currentIndex == positions.length) {
					dispose();
					mainApp.configFrameClosed(positions);
					// for (int i = 0; i < posis.length; i++) {
					// System.out.println(posis[i].toString());
					// }
				}
				repaint();

			}

		});

		this.setVisible(true);
	}

	/**
	 * Closes this window without sending the current values.
	 */
	public void close() {
		this.dispose();
	}

	/**
	 * Paints the frame just like we need it.
	 */
	@Override
	public void paint(Graphics g) {
		Color red = new Color(255, 0, 0, 128);
		Color blue = new Color(0, 0, 255, 128);

		int old = currentIndex - 1;
		g.drawImage(background, 0, 0, null);

		String whichPartToChoose;

		// Already chosen area is being painted.
		if (currentIndex > 1) {
			g.setColor(red);
			// Left side of the screen
			g.fillRect((int) positions[0].getX(), (int) positions[0].getY(),
					(int) (positions[1].getX() - positions[0].getX()),
					(int) (positions[1].getY() - positions[0].getY()));
			// Right side of the screen
			g.fillRect(screenWidth - (int) positions[0].getX(),
					(int) positions[1].getY(),
					(int) (positions[0].getX() - positions[1].getX()),
					(int) (positions[0].getY() - positions[1].getY()));
			whichPartToChoose = "Choose upper part";
		} else {
			whichPartToChoose = "Choose side part";
		}

		// First rectangle will be red, second blue.
		g.setColor(currentIndex < 2 ? red : blue);

		// Current mouse position lines

		// Left side of the screen
		g.drawLine(0, (int) mousePos.getY(), (int) screenWidth,
				(int) mousePos.getY());
		g.drawLine((int) mousePos.getX(), 0, (int) mousePos.getX(),
				screenHeight);
		// Right side of the screen
		g.drawLine(screenWidth - (int) mousePos.getX(), 0, screenWidth
				- (int) mousePos.getX(), screenHeight);

		if (currentIndex % 2 == 1) {

			// Upper left corner (old corner)

			// Left side
			g.drawLine(0, (int) positions[old].getY(), (int) screenWidth,
					(int) positions[old].getY());
			g.drawLine((int) positions[old].getX(), 0,
					(int) positions[old].getX(), screenHeight);

			// Right side of the screen
			g.drawLine(screenWidth - (int) positions[old].getX(), 0,
					screenWidth - (int) positions[old].getX(), screenHeight);

			// Left screen side rectangle
			g.fillRect((int) positions[old].getX(),
					(int) positions[old].getY(),
					(int) (mousePos.getX() - positions[old].getX()),
					(int) (mousePos.getY() - positions[old].getY()));

			// Right screen side rectangle
			g.fillRect(screenWidth - (int) mousePos.getX(),
					(int) positions[old].getY(),
					(int) (mousePos.getX() - positions[old].getX()),
					(int) (mousePos.getY() - positions[old].getY()));
		}

		// Scala

		g.setColor(Color.YELLOW);

		// X Coordinates
		for (int i = 0; i < (int) screenWidth; i += 50) {
			if (i % 100 == 0) {
				g.drawLine(i, 0, i, 20);
				g.drawString("" + i, i, 20);
			} else {
				g.drawLine(i, 0, i, 10);

			}
		}

		// Y Coordinates
		for (int i = 0; i < (int) screenHeight; i += 50) {
			if (i % 100 == 0) {
				g.drawLine(0, i, 20, i);
				g.drawString("" + i, 0, i);
			} else {
				g.drawLine(0, i, 10, i);

			}
		}
		// A Place to write something
		g.setColor(Color.BLACK);
		g.fillRect((int) ((screenWidth / 2) - 75), screenHeight - 75, 150, 75);

		// X and Y Coordinate of the current mouse position
		g.setColor(Color.WHITE);
		g.drawString("x=" + (int) (mousePos.getX() + 1) + ", y="
				+ (int) (mousePos.getY() + 1), (int) ((screenWidth / 2) - 72),
				screenHeight - 35);

		// Number of pixels in the currently chosen rectangle.
		if (currentIndex % 2 == 1) {
			// Only length of x times length of y.
			g.drawString(
					"pixels="
							+ (int) ((mousePos.getX() - positions[old].getX()) * (mousePos
									.getY() - positions[old].getY())),
					(int) ((screenWidth / 2) - 72), screenHeight - 15);
		}

		// Tell user which part to choose
		g.setColor(Color.WHITE);
		g.drawString(whichPartToChoose, (int) ((screenWidth / 2) - 72),
				screenHeight - 55);

	}
}