package de.zucizu.ledlights;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;

/**
 * 
 * This window is for testing purposes.
 * 
 * @author zucizu
 * 
 */
public class TestFrame extends JFrame {

	/**
	 * Paint left rectangle.
	 */
	private static final int RECTANGLE_LEFT = 0;

	/**
	 * Paint upper left rectangle.
	 */
	private static final int RECTANGLE_UPPER_LEFT = 1;

	/**
	 * Paint upper right rectangle.
	 */
	private static final int RECTANGLE_UPPER_RIGHT = 2;

	/**
	 * Paint right rectangle.
	 */
	private static final int RECTANGLE_RIGHT = 3;

	/**
	 * Indicates which of the rectangles will be painted.
	 */
	private int currentRectangle = 0;

	/**
	 * The color to be painted (index in RGB_COLORS).
	 */
	private int currentColor = 0;

	/**
	 * An array that contains RGB.
	 */
	private static final Color[] RGB_COLORS = { Color.RED, Color.GREEN,
			Color.BLUE };

	/**
	 * Size for full screen.
	 */
	private Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

	/**
	 * Width of screen.
	 */
	private int screenWidth = (int) screenSize.getWidth();

	/**
	 * Height of screen.
	 */
	private int screenHeight = (int) screenSize.getHeight();

	/**
	 * Forces the update timer to stop.
	 */
	private boolean cancelUpdatetimer;

	/**
	 * The parent component to inform when closed.
	 */
	private AmbientLED mainApp;

	/**
	 * 
	 */
	private double ratio = 3;

	/**
	 * Created by Eclipse.
	 */
	private static final long serialVersionUID = -6511305716423056584L;

	/**
	 * Creates a new full screen frame to test the LED lights.
	 */
	public TestFrame(AmbientLED mainApp) {
		super();

		this.mainApp = mainApp;

		// Full screen.
		this.setSize(this.screenSize);
		this.setUndecorated(true);

		// To close this frame.
		this.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent e) {
			}

			// Any key closes this frame.
			public void keyPressed(KeyEvent e) {
				close();
			}

			public void keyReleased(KeyEvent e) {

			}
		});

		// To close this frame.
		this.addMouseListener(new MouseListener() {

			// A mouse click will close this frame.
			public void mousePressed(MouseEvent e) {
				close();
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseClicked(MouseEvent e) {
			}

			public void mouseReleased(MouseEvent e) {
			}
		});

		// The cursor would only manipulate the mean value. So we make an empty
		// one.
		BufferedImage cursorImg = new BufferedImage(1, 1,
				BufferedImage.TYPE_INT_ARGB);
		Cursor nullCursor = Toolkit.getDefaultToolkit().createCustomCursor(
				cursorImg, new Point(0, 0), "null");
		this.getContentPane().setCursor(nullCursor);

		// This thread updates the states to be painted.
		Thread updateTimer = new Thread(new Runnable() {

			@Override
			public void run() {
				while (!cancelUpdatetimer) {
					try {
						Thread.sleep(1500);
					} catch (InterruptedException e) {
					}
					currentRectangle = (++currentRectangle) % 4;
					if (currentRectangle == 0) {
						currentColor = (++currentColor) % 3;
					}
					repaint();
				}
			}
		});
		updateTimer.start();

		this.setVisible(true);
	}

	/**
	 * Stops the update timer and closes this frame.
	 */
	public void close() {
		mainApp.testFrameClosed();
		this.cancelUpdatetimer = true;
		this.dispose();
	}

	@Override
	public void paint(Graphics g) {
		int xPos, yPos, sizeX, sizeY;
		xPos = yPos = sizeX = sizeY = 0;

		// Black background.
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, this.screenWidth, this.screenHeight);

		// Color to be painted.
		Color rectangleColor = RGB_COLORS[this.currentColor];

		switch (this.currentRectangle) {
		case (TestFrame.RECTANGLE_LEFT):
			xPos = 0;
			yPos = 0;
			sizeX = (int) (this.screenWidth / ratio);
			sizeY = this.screenHeight;
			break;

		case (TestFrame.RECTANGLE_RIGHT):
			xPos = (int) ((this.screenWidth / ratio) * (ratio - 1));
			yPos = 0;
			sizeX = (int) (this.screenWidth / ratio);
			sizeY = this.screenHeight;
			break;

		case (TestFrame.RECTANGLE_UPPER_LEFT):
			xPos = 0;
			yPos = 0;
			sizeX = (int) (this.screenWidth / 2);
			sizeY = (int) (this.screenHeight / ratio);
			break;

		case (TestFrame.RECTANGLE_UPPER_RIGHT):
			xPos = (int) (this.screenWidth / 2);
			yPos = 0;
			sizeX = (int) (this.screenWidth / 2);
			sizeY = (int) (this.screenHeight / ratio);
			break;

		}
		g.setColor(rectangleColor);
		g.fillRect(xPos, yPos, sizeX, sizeY);
	}
}