package de.zucizu.ledlights;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

/**
 * Class represents a waiting window that may be used in different contexts.
 * 
 * @author zucizu
 * 
 */
public class WaitingFrame implements Runnable {

	/**
	 * Width of the screen to center this window.
	 */
	private int screenWidth;

	/**
	 * @see screenWidth
	 */
	private int screenHeight;

	/**
	 * This label holds the text to be displayed.
	 */
	private JLabel textLabel = null;

	/**
	 * Frame do display everything.
	 */
	private JFrame mainFrame;

	/**
	 * This constructor sets all local variables. To show the window call run()
	 * on a new Thread holding this class.
	 * 
	 * @see WaitingFrame.run
	 * 
	 * @param screenWidth
	 *            Width of the screen (needed to center this window).
	 * @param screenHeight
	 *            Height of the screen (needed to center this window).
	 * @param text
	 *            Text to be displayed (can be changed later).
	 */
	public WaitingFrame(int screenWidth, int screenHeight, String text) {
		this.textLabel = new JLabel(text);
		this.screenWidth = screenWidth;
		this.screenHeight = screenHeight;
	}

	/**
	 * Shows and centers the waiting window.
	 */
	@Override
	public void run() {
		Font f = new Font("Dialog", Font.PLAIN, 24);
		textLabel.setFont(f);
		textLabel.setBackground(Color.BLACK);
		textLabel.setForeground(Color.WHITE);
		mainFrame = new JFrame();
		JProgressBar jp = new JProgressBar();
		jp.setForeground(Color.WHITE);
		jp.setBackground(Color.BLACK);
		jp.setIndeterminate(true);
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(textLabel, BorderLayout.NORTH);
		mainPanel.add(jp, BorderLayout.SOUTH);
		mainPanel.setBackground(Color.BLACK);
		mainPanel.setBorder(BorderFactory.createLineBorder(Color.RED, 4));
		mainFrame.getContentPane().add(mainPanel);
		mainFrame.setUndecorated(true);
		mainFrame.pack();
		// This windows shall not be closed.
		mainFrame.setDefaultCloseOperation(0);
		mainFrame.setAlwaysOnTop(true);
		mainFrame.setLocation((int) (screenWidth - mainFrame.getWidth()) / 2,
				(int) (screenHeight - mainFrame.getHeight()) / 2);
		mainFrame.setVisible(true);
	}

	/**
	 * Forces the waiting window to dispose.
	 */
	public void close() {
		mainFrame.dispose();
	}

	/**
	 * Changes the text displayed to the user.
	 * 
	 * @param text
	 *            The new text to show.
	 */
	public void changeText(String text) {
		textLabel.setText(text);
	}

}
